/* 
 *  libtfflt.c:
 *
 * Copyright (c) 1998,2001 Hajime Mochizuki <motizuki@jaist.ac.jp>
 * Natural Language Processing Laboratory,
 * Japan Advanced Institute of Science and Technology.
 * 
 * Use of this software is granted only for RESEARCH PURPOSES. 
 * Redistribution is granted as long as all copyright notices are retained.
 * Redistribution with modification is allowed provided that you make your 
 * modified version obviously distinguishable from the original one. 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR 
 * IMPLIED WARRANTIES ARE DISCLAIMED.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* for CHAR_BIT */
#if defined(BSD44) || defined(DIGITAL)
#include <machine/limits.h>
#else
#include <limits.h>
#endif

#include "cmn.h"
#include "chafil.h"
#include "tfflt.h"

void init_hash_f (hashtab, hashsize)
     struct hash_entry *hashtab[];
     int    hashsize;
{
	int i;
	for (i=0; i<hashsize;i++) {
		*(hashtab+i)=NULL;
	}
}

void init_hash (hashtab, hashsize, sw, outformat, df)
     struct hash_entry *hashtab[];
     int hashsize;
     int sw;
     int outformat;
     FILE *df;
{
	int i;
	struct hash_entry *hp, *hpp;

	i=0;
	for (i=0; i<hashsize; i++) {
		hp=*(hashtab+i);
		while (hp!=NULL) {
			if (sw==9) {
				;
			} else if (sw==0) {
				if (hp->tf>0) {
					if (outformat==1) {
						printf ("%d %s\n", hp->tf, hp->word);
					} else {
						printf ("%s %d\n", hp->word, hp->tf);
					}
					hp->tf=0;
				}
			} else {
				if (hp->df>0) {
					if (outformat==1) {
						fprintf (df, "%d %s\n", hp->df, hp->word);
					} else {
						fprintf (df, "%s %d\n", hp->word, hp->df);
					}
				}
			}
			hp=hp->next;
		}
	}
}

void sort_qryvec (first, last, a)
      int first, last;
      struct hash_entry a[];
{
	int i, j, x;
	struct hash_entry s;
	
	x = a[((first+last)/2)].num;
        i = first; j = last;
        for (;;) {
                while (a[i].num < x) i++;
                while (x < a[j].num) j--;
                if (i>=j) break;
                s    = a[i];
                a[i] = a[j];
                a[j] = s;
                i++, j--;
        }
        if (first < i-1) sort_qryvec (first, i-1, a);
        if (j+1  < last) sort_qryvec (j+1,  last, a);
}

struct hash_entry *get_qryvec (hashtab, hashsize, entry)
     struct hash_entry *hashtab[];
     int hashsize;
     int entry;
{
	int i, j;
	struct hash_entry *qvec, *hp, *hpp;

	if ((qvec = calloc((size_t)entry, sizeof(struct hash_entry)))==NULL) {
		perror("qvec");
		exit(1);
	}

	i=j=0;
	for (i=0; i<hashsize; i++) {
		hp=*(hashtab+i);
		while (hp!=NULL) {
			if (hp->tf>0) {
				memmove(&qvec[j], hp, sizeof(struct hash_entry));
				hp->tf=0;
				hp->df=0;
				hp->num=0;
				hp->qscr=(double)0;
				j++;
				if (j>=entry) {
					return qvec;
				}
			}
			hpp = hp;
			hp = hpp->next;
			free(hpp); hpp=NULL;
		}
		*(hashtab+i)=NULL;
	}
	return qvec;
}

unsigned int hash (s, hashsize)
     char *s;
     int  hashsize;
{
        unsigned int val;

        for (val=0; *s!='\0'; s++) {
                val = ((val<<CHAR_BIT)+*s) % hashsize;
        }
        return val;
}


int hash_update (word, num, hashtab, hashsize)
    char *word;
    int  num;
    struct hash_entry *hashtab[];
    int  hashsize;
{
	unsigned int i;
	struct hash_entry *hp;
	
	i = hash (word, hashsize);
	if (*(hashtab+i)!=NULL) {
		for (hp=*(hashtab+i); hp!=NULL; hp=hp->next) {
			if (strcmp(word, hp->word)==0) {
				hp->tf++;
				if (hp->tf==1) {
					hp->df++;
					hp->num = num;
					return 1;
				} else {
					return 0;
				}
			}
		}
	}
	if ((hp=calloc((size_t)1, sizeof(*hp)))==NULL) {
		fprintf (stderr, "calloc error\n");
		exit(1);
	}
	hp->word = strdup(word);
	hp->tf   = 1;
	hp->df   = 1;
	hp->num  = num;
	hp->next = *(hashtab+i);
	*(hashtab+i)=hp;
	return 1;
}

int tffilter (words, hashtab, hashsize)
     CHAFLT_OUT *words;
     struct hash_entry *hashtab[];
     int    hashsize;
{
	int i, entry;

	entry=0;
	for (i=0; i<chaflt_out_num; i++) {
		entry += hash_update(words[i].term,
				     words[i].number,
				     hashtab, hashsize);
/*		free(words[i].term);words[i].term=NULL;*/
	}
/*	free(words);*/
/*	chaflt_out_num=0;*/
	return entry;
}
